Program Beacon;

#pragma CLOCK_FREQ 2457600                        // Change to match xtal frequency

ASM __config _XT_OSC & _WDT_OFF & _PWRTE_OFF & _CP_OFF

// Pin assignment
//   Port A
//     0 : PTT
//     1 : Key1
//     2 : Key2
//     3 : Key3
//     4 : Key4
//
//   Port B                 DIP             DIP On          DIP Off
//     0 : Duty Cycle       4               0 = No pause    1 = Pause
//     1 : Locator          3               0 = JO75IC      1 = JO75KC
//     2 : Send Locator     2               0 = No          1 = Yes
//     3 : Call             1               0 = OZ4UHF/A    1 = OZ4UHF
//     4 : Pause duration   5               0 = 5 s         1 = 1 s
//     5 : QSK (Not impl.)  6               0 = Yes         1 = No
//     6 : Low Power        7               0 = Yes         1 = No
//     7 :                  8 

Const
  PTTPause   =50;                                 // Pause between PTT and Key [ms]
  DitDuration=100;                                // Duration of a dit [ms], 100 ms is equal to 60 LPM
  Rounds     =10;                                 // Number of rounds before Low Power Indicator (RA0) is unset
  
Var
  I,J,PA_On,PA_Off:char;
  
Procedure SendDit(Aon:char;Aoff:char);

begin
  Output_port_A(Aon);
  Delay_ms(DitDuration);
  Output_port_A(Aoff);
  Delay_ms(DitDuration);
end;

Procedure SendDash(Aon:char;Aoff:char);

begin
  Output_port_A(Aon);
  Delay_ms(DitDuration);
  Delay_ms(DitDuration);
  Delay_ms(DitDuration);
  Output_port_A(Aoff);
  Delay_ms(DitDuration);
end;

Procedure CharacterPause;

begin                                             // Pause is actually 3 x dit but last dit in Send procs.
  Delay_ms(DitDuration);
  Delay_ms(DitDuration);
end;

Begin
  Disable_Interrupt('GIE');                       // Initialization of registers
  Clear_Bit('OPTION_REG',7);                      // Enable RB pull up resistors
  Set_Tris_A(00000b);
  Set_Tris_B(11111111b);
  Output_port_A(0);                               // Set all outputs low

  Output_high_port_A(0);                          // PTT On
  Delay_ms(PTTPause);                                    
  
  I:=14;
  J:=0;

  While 1 do
  begin                                           // Key bit pattern
    If I=14 then I:=28; else                      // First TX  11100 = 28
    If I=28 then I:=26; else                      // Second TX 11010 = 26
    If I=26 then I:=22; else                      // Third TX  10110 = 22
    If I=22 then I:=14;                           // Fourth TX 01110 = 14

    J:=J+1;                                       // Rounds control
    If J=4*Rounds+5 then
      J:=0;

    If J>4*Rounds then
    begin                                         // Low Power mode
      PA_On:=11110b;
      PA_Off:=I;
    end;
    else
    begin                                         // High power mode
      PA_On:=11111b;
      PA_Off:=I+1;
    end;

    If Input_pin_port_B(6)=1 then                 // Low Power 0 = No, 1 = Yes
    begin                                         // No Low Power mode so set to high power
      PA_On:=11111b;
      PA_Off:=I+1;
    end;

    If Input_pin_port_B(0)=1 then                 // Pause: 1 = Pause, 0 = No pause
    begin
      Output_port_A(PA_On);                       // Key On
      If Input_pin_port_B(4)=0  then
        Delay_s(5);                               // Send carrier
      else
        Delay_s(1);                               // Send carrier
    end;
    Output_port_A(PA_Off);                        // Stop sending carrier

    Delay_ms(250);                                // 500 ms pause between keying and ID
    Delay_ms(250);

    SendDash(PA_On,PA_Off);                       // Send O, ---
    SendDash(PA_On,PA_Off);
    SendDash(PA_On,PA_Off);
    
    CharacterPause;                               // Inter character pause
    
    SendDash(PA_On,PA_Off);                       // Send Z, --**
    SendDash(PA_On,PA_Off);
    SendDit(PA_On,PA_Off);
    SendDit(PA_On,PA_Off);
    
    CharacterPause;                               // Inter character pause
    
    SendDit(PA_On,PA_Off);                        // Send 4, ****-
    SendDit(PA_On,PA_Off);
    SendDit(PA_On,PA_Off);
    SendDit(PA_On,PA_Off);
    SendDash;
    
    CharacterPause;                               // Inter character pause
    
    SendDit(PA_On,PA_Off);                        // Send U, **-
    SendDit(PA_On,PA_Off);
    SendDash;

    CharacterPause;                               // Inter character pause

    SendDit(PA_On,PA_Off);                        // Send H, ****
    SendDit(PA_On,PA_Off);
    SendDit(PA_On,PA_Off);
    SendDit(PA_On,PA_Off);
        
    CharacterPause;                               // Inter character pause

    SendDit(PA_On,PA_Off);                        // Send F, **-*
    SendDit(PA_On,PA_Off);
    SendDash(PA_On,PA_Off);
    SendDit(PA_On,PA_Off);
    
    If Input_pin_port_B(3)=0 then                 // If 0 then call = OZ4UHF/A
    begin
      CharacterPause;                             // Inter character pause

      SendDash(PA_On,PA_Off);                     // Send /, -**-*
      SendDit(PA_On,PA_Off);
      SendDit(PA_On,PA_Off);
      SendDash(PA_On,PA_Off);
      SendDit(PA_On,PA_Off);
    
      CharacterPause;                             // Inter character pause
      
      SendDit(PA_On,PA_Off);                      // Send A, *-
      SendDash(PA_On,PA_Off);
    end;

    If Input_pin_port_B(2)=1 then                 // Send locator if 1
    begin
      CharacterPause;                             // Word pause, 7 x dit
      CharacterPause;                             
      CharacterPause;                             
      
      SendDit(PA_On,PA_Off);                      // Send J, *---
      SendDash(PA_On,PA_Off);
      SendDash(PA_On,PA_Off);
      SendDash(PA_On,PA_Off);

      CharacterPause;                             // Inter character pause
            
      SendDash(PA_On,PA_Off);                     // Send O, ---
      SendDash(PA_On,PA_Off);
      SendDash(PA_On,PA_Off);
      
      CharacterPause;                             // Inter character pause
      
      SendDash(PA_On,PA_Off);                     // Send 7, --***
      SendDash(PA_On,PA_Off);
      SendDit(PA_On,PA_Off);
      SendDit(PA_On,PA_Off);
      SendDit(PA_On,PA_Off);
              
      CharacterPause;                             // Inter character pause

      SendDit(PA_On,PA_Off);                      // Send 5, *****
      SendDit(PA_On,PA_Off);
      SendDit(PA_On,PA_Off);
      SendDit(PA_On,PA_Off);
      SendDit(PA_On,PA_Off);
   
      CharacterPause;                             // Inter character pause

      If Input_pin_port_B(1)=1 then               // Send JO75KC if 1, JO75IC if 0
      begin
        SendDash(PA_On,PA_Off);                   // Send K, -*-
        SendDit(PA_On,PA_Off);                                  
        SendDash(PA_On,PA_Off);
    
        CharacterPause;                           // Inter character pause
    
        SendDash(PA_On,PA_Off);                   // Send C, -*-*
        SendDit(PA_On,PA_Off);                                  
        SendDash(PA_On,PA_Off);
        SendDit(PA_On,PA_Off);                                  
      end;
      else
      begin
        SendDit(PA_On,PA_Off);                    // Send I, **
        SendDit(PA_On,PA_Off);                                  
    
        CharacterPause;                           // Inter character pause
    
        SendDash(PA_On,PA_Off);                   // Send C, -*-*
        SendDit(PA_On,PA_Off);                                  
        SendDash(PA_On,PA_Off);
        SendDit(PA_On,PA_Off);                                  
      end;
    end;

    Delay_ms(250);                                // 500 ms pause between ID and keying
    Delay_ms(250);
  end;
End.